
const argv = require('minimist')(process.argv.slice(2));
const log = require('console').log;
const readlineSync = require('readline-sync');
const clipboard = require('clipboardy');
const shell = require('shelljs');

const fs = require('fs');
const path = require('path');

const digger = require('./digger.js');
const parser = require('./parser.js');
const getter = require('./getter.js');
const cssDigger = require('./cssDigger.js');
const cssParser = require('./cssParser.js');

const cfg = require('./configs/config.json');
const en = require('./configs/en.json');
const cn = require('./configs/cn.json');

let langCfg = argv['lang'] ? getLang(argv['lang']) : getLang('en');
if (!langCfg) {
    langCfg = getLang('en');
}

if (argv['help']) {
    logContent(langCfg.intro);
    process.exit(1);
}

const name = argv['name'] ? argv['name'] : '';
const verbose = argv['verbose'] ? !!argv['verbose'] : true;

log(langCfg.help);
log(langCfg.lang);
log(langCfg.name);
log(langCfg.verbose);

waitAnyKey(langCfg.askURL);
const url = getFromClipboard();
waitAnyKey(langCfg.askContent);
let content = getFromClipboard();

const dest = genIdentifier();
shell.exec('md ' + dest);

digger.config(cfg.html, content);
digger.dig();
const originalAddrList = digger.originalAddrList();

parser.config(cfg.html, dest, getDomain(url), originalAddrList);
parser.parse();

const done = getter.downloadAllOfThese(verbose, 'FROM HTML: ', parser.downloadAddrList(), parser.saveAddrList());

makeHtmlFile(name, content);

done.then(() => {
    fs.readdirSync(dest).map(function(file) {
        if (file.substr(file.lastIndexOf('.')) === '.css') {
            let content = fs.readFileSync(path.join(dest, file)).toString();

            cssDigger.reset();
            cssDigger.config(cfg.css, content);
            cssDigger.dig();
            const originalAddrList = cssDigger.originalAddrList();

            cssParser.reset();
            cssParser.config(cfg.css, dest, path.join(dest, file), parser.cssBox(), originalAddrList);
            cssParser.parse();

            if (cssParser.downloadAddrList().length > 0) {
                getter.downloadAllOfThese(true, 'FROM CSS: ', cssParser.downloadAddrList(), cssParser.saveAddrList());

                makeCssFile(file, content, originalAddrList, cssParser.replaceAddrList());
            }
        }
    })
})

function getLang(lang) {
    return {'en': en, 'cn': cn}[lang];
}

function logContent(content) {
    content.forEach((item) => {
        log(item);
    });
}

function waitAnyKey(queryText) {
    readlineSync.question(queryText);
}

function getFromClipboard() {
    return clipboard.readSync();
}

function getDomain(url) {
    let domain = '';

    if (url.substr(0, 7) != 'http://' && url.substr(0, 8) != 'https://') {
        throw new Error();
    }
    url = url.substr(url.length - 1) === '/' ? url : url + '/';

    if (url.substr(0, 7) === 'http://') {
        url = url.substr(7);
        domain = 'http://' + url.substr(0, url.indexOf('/'));
    } else {
        url = url.substr(8);
        domain = 'https://' + url.substr(0, url.indexOf('/'));
    }

    return domain;
}

function genIdentifier() {
    const d = new Date();

    return d.getFullYear().toString() + ifNeedZero(d.getMonth() + 1) + ifNeedZero(d.getUTCDate()) + ifNeedZero(d.getHours().toString()) + ifNeedZero(d.getMinutes().toString()) + ifNeedZero(d.getSeconds().toString()); 
}

function ifNeedZero(number) {
    return number < 10 ? "0" + number.toString() : number.toString();
}

function replace(content, originalAddrList, replaceAddrList) {
    originalAddrList.forEach((addr, idx) => {
        addr = addr.replace(new RegExp('\\?', 'g'), '\\?');
        content = content.replace(new RegExp(addr, 'g'), replaceAddrList[idx]);
    });

    return content;
}

function makeFile(name, content) {
    const stream = fs.createWriteStream(name);
    stream.write(content);
    stream.end();
}

function makeHtmlFile(name, content) {
    if (cfg.killScript) {
        content = content.replace(new RegExp('<script.*?</script>', 'g'), '');
        content = content.replace(new RegExp('<script.*?/>', 'g'), '');
        content = content.replace(new RegExp('<script', 'g'), '<textarea style="display:none" ');
        content = content.replace(new RegExp('</script>', 'g'), '</textarea>');
    }

    content = replace(content, originalAddrList, parser.replaceAddrList());
    makeFile(name + '_' + dest + '.html', '<!-- Current URL: ' + url + '-->\r\n' + content);
}

function makeCssFile(name, content, originalArr, replaceArr) {
    fs.renameSync(path.join(dest, name), path.join(dest, name + '.xss'));

    content = replace(content, originalArr, replaceArr);
    makeFile(path.join(dest, name), content);
}